@@ -47,6 +47,7 @@ static void tpm2_alg_util_for_each_alg(alg_iter iterator, void *userdata) {
         { .name = "ecc", .id = TPM2_ALG_ECC, .flags = tpm2_alg_util_flags_asymmetric|tpm2_alg_util_flags_base },
 
         // Symmetric
+        { .name = "tdes", .id = TPM2_ALG_TDES, .flags = tpm2_alg_util_flags_symmetric },
         { .name = "aes", .id = TPM2_ALG_AES, .flags = tpm2_alg_util_flags_symmetric },
         { .name = "camellia", .id = TPM2_ALG_CAMELLIA, .flags = tpm2_alg_util_flags_symmetric },
 
@@ -193,9 +194,7 @@ static alg_parser_rc handle_scheme_sign(const char *scheme,
     bool do_scheme_hash_alg = false;
 
     if (public->publicArea.type == TPM2_ALG_ECC) {
-        if (!strncmp(scheme, "oaep", 4)) {
-            do_scheme_halg(scheme, 4, TPM2_ALG_OAEP);
-        } else if (!strncmp(scheme, "ecdsa", 5)) {
+        if (!strncmp(scheme, "ecdsa", 5)) {
             do_scheme_halg(scheme, 5, TPM2_ALG_ECDSA);
         } else if (!strncmp(scheme, "ecdh", 4)) {
             do_scheme_halg(scheme, 4, TPM2_ALG_ECDH);
@@ -236,6 +235,8 @@ static alg_parser_rc handle_scheme_sign(const char *scheme,
             do_scheme_halg(scheme, 6, TPM2_ALG_RSAPSS);
         } else if (!strncmp("rsassa", scheme, 6)) {
             do_scheme_halg(scheme, 6, TPM2_ALG_RSASSA);
+        } else if (!strncmp(scheme, "oaep", 4)) {
+            do_scheme_halg(scheme, 4, TPM2_ALG_OAEP);
         }
     }
 
@@ -594,6 +595,32 @@ bool tpm2_alg_util_handle_ext_alg(const char *alg_spec, TPM2B_PUBLIC *public) {
     return false;
 }
 
+tool_rc tpm2_alg_util_handle_rsa_ext_alg(const char *alg_spec,
+    TPM2B_PUBLIC *public) {
+
+    #define RSA_KEYBITS_STRLEN 6
+    char *ext_alg_str = calloc(1, strlen(alg_spec) + strlen("rsa") +
+        RSA_KEYBITS_STRLEN);
+
+    strcat(ext_alg_str, "rsa");
+    switch(public->publicArea.parameters.rsaDetail.keyBits) {
+        case 1024: strcat(ext_alg_str, "1024:");
+                   break;
+        case 2048: strcat(ext_alg_str, "2048:");
+                   break;
+        case 3072: strcat(ext_alg_str, "3072:");
+                   break;
+        case 4096: strcat(ext_alg_str, "4096:");
+                   break;
+    };
+    strcat(ext_alg_str, alg_spec);
+
+    bool result = tpm2_alg_util_handle_ext_alg(ext_alg_str, public);
+    free(ext_alg_str);
+
+    return result ? tool_rc_success : tool_rc_general_error;
+}
+
 static alg_iter_res find_match(TPM2_ALG_ID id, const char *name,
         tpm2_alg_util_flags flags, void *userdata) {
 
@@ -981,6 +1008,7 @@ tool_rc tpm2_alg_util_public_init(char *alg_details, char *name_halg, char *attr
     TPM2B_PUBLIC tmp = *public;
     bool res = tpm2_alg_util_handle_ext_alg(alg_details, &tmp);
     if (!res) {
+        LOG_ERR("Could not handle algorithm: \"%s\"", alg_details);
         return tool_rc_unsupported;
     }
 
