@@ -1,5 +1,7 @@
 #include <inttypes.h>
 #include <stdlib.h>
+#include <string.h>
+
 #include <tss2/tss2_tpm2_types.h>
 
 #include "log.h"
@@ -117,7 +119,7 @@ bool parse_event2body(TCG_EVENT2 const *event, UINT32 type) {
             }
 
             if (event->EventSize < sizeof(*data) + data->UnicodeNameLength *
-                sizeof(char16_t) + data->VariableDataLength)
+                sizeof(UTF16_CHAR) + data->VariableDataLength)
             {
                 LOG_ERR("size is insufficient for UEFI variable data");
                 return false;
@@ -271,7 +273,8 @@ bool foreach_sha1_log_event(tpm2_eventlog_context *ctx, TCG_EVENT const *eventhd
 
         /* event data callback */
         if (ctx->event2_cb != NULL) {
-            ret = ctx->event2_cb(event, eventhdr->eventType, ctx->data);
+            ret = ctx->event2_cb(event, eventhdr->eventType, ctx->data,
+                                 ctx->eventlog_version);
             if (ret != true) {
                 return false;
             }
@@ -281,6 +284,112 @@ bool foreach_sha1_log_event(tpm2_eventlog_context *ctx, TCG_EVENT const *eventhd
     return true;
 }
 
+/*
+ * For event types where digest can be verified from their event payload,
+ * perform verification to ensure event payload was not tempered
+ */
+bool verify_digests(size_t eventnum, TCG_EVENT_HEADER2 const *eventhdr, TCG_EVENT2 *event) {
+
+size_t i;
+
+    TCG_DIGEST2 const *digest = eventhdr->Digests;
+    UINT32 digest_count = eventhdr->DigestCount;
+    UINT32 event_type = eventhdr->EventType;
+    switch (event_type) {
+    /* Digests of these event types are calculated directly from event->Event, thus can be verified  */
+    case EV_S_CRTM_VERSION:
+    case EV_SEPARATOR:
+    case EV_EFI_VARIABLE_DRIVER_CONFIG:
+    case EV_EFI_GPT_EVENT:
+        for (i = 0; i < digest_count; i++) {
+            TPMI_ALG_HASH alg = digest->AlgorithmId;
+            TPM2B_DIGEST calc_digest;
+
+            bool result = tpm2_openssl_hash_compute_data(alg, event->Event,
+            event->EventSize, &calc_digest);
+            if (!result) {
+                LOG_WARN("Event %zu: Cannot calculate hash value from data", eventnum - 1);
+                return false;
+            }
+
+            size_t alg_size = tpm2_alg_util_get_hash_size(alg);
+            if (memcmp(calc_digest.buffer, digest->Digest, alg_size) != 0) {
+                LOG_WARN("Event %zu's digest does not match its payload", eventnum - 1);
+                return false;
+            }
+
+            digest = (TCG_DIGEST2*)((uintptr_t)digest->Digest + alg_size);
+        }
+        break;
+
+    /* Shim and grub use this event type for various tasks */
+    case EV_IPL:
+        /* PCR9: used to measure loaded kernel and initramfs images which cannot
+           be verified from eventlog alone */
+        if (eventhdr->PCRIndex == 9) {
+            return true;
+        }
+
+        /* PCR14: used to measure MokList, MokListX, and MokSBState which cannot
+           be verified from eventlog alone */
+        if (eventhdr->PCRIndex == 14) {
+            return true;
+        }
+
+        /* PCR8: used to measure grub and kernel command line */
+        if (eventhdr->PCRIndex != 8) {
+            LOG_WARN("Event %zu is unexpectedly not extending either PCR 8, 9, or 14", eventnum - 1);
+            return false;
+        }
+
+        /* Digest is applied on the string between "^[a-zA-Z_]+:? " and EOL,
+         * including or excluding the trailing NULL character */
+        for (i = 0; i < digest_count; i++) {
+            size_t j;
+
+            for (j = 0; j < event->EventSize; j++) {
+                if (event->Event[j] == ' ')
+                    break;
+            }
+
+            if (j + 1 >= event->EventSize || event->Event[event->EventSize - 1] != '\0') {
+                LOG_WARN("Event %zu's event data is in unexpected format", eventnum - 1);
+                return false;
+            }
+
+            TPM2B_DIGEST calc_digest;
+            TPMI_ALG_HASH alg = digest->AlgorithmId;
+            /* First try to calculate the hash excluding the trailing \0 */
+            bool result = tpm2_openssl_hash_compute_data(alg,
+            event->Event + (j + 1), event->EventSize - (j + 2), &calc_digest);
+            if (!result) {
+                LOG_WARN("Event %zu: Cannot calculate hash value from data", eventnum - 1);
+                return false;
+            }
+
+            size_t alg_size = tpm2_alg_util_get_hash_size(alg);
+            if (memcmp(calc_digest.buffer, digest->Digest, alg_size) != 0) {
+                /* Next try to calculate the hash including the trailing \0 */
+                bool result = tpm2_openssl_hash_compute_data(alg,
+                event->Event + (j + 1), event->EventSize - (j + 1), &calc_digest);
+                if (!result) {
+                    LOG_WARN("Event %zu: Cannot calculate hash value from data", eventnum - 1);
+                    return false;
+                }
+
+                if (memcmp(calc_digest.buffer, digest->Digest, alg_size) != 0) {
+                    LOG_WARN("Event %zu's digest does not match its payload", eventnum - 1);
+                    return false;
+                }
+            }
+            digest = (TCG_DIGEST2*)((uintptr_t)digest->Digest + alg_size);
+        }
+        break;
+    }
+
+    return true;
+}
+
 bool foreach_event2(tpm2_eventlog_context *ctx, TCG_EVENT_HEADER2 const *eventhdr_start, size_t size) {
 
     if (eventhdr_start == NULL) {
@@ -328,9 +437,14 @@ bool foreach_event2(tpm2_eventlog_context *ctx, TCG_EVENT_HEADER2 const *eventhd
             return ret;
         }
 
+        /* digest verification */
+        if (ctx->data != 0) {
+            verify_digests(*(size_t*)ctx->data, eventhdr, event);
+        }
+
         /* event data callback */
         if (ctx->event2_cb != NULL) {
-            ret = ctx->event2_cb(event, eventhdr->EventType, ctx->data);
+            ret = ctx->event2_cb(event, eventhdr->EventType, ctx->data, ctx->eventlog_version);
             if (ret != true) {
                 return false;
             }
@@ -421,7 +535,11 @@ bool specid_event(TCG_EVENT const *event, size_t size,
 
 bool parse_eventlog(tpm2_eventlog_context *ctx, BYTE const *eventlog, size_t size) {
 
-    if(!eventlog || (size < sizeof(TCG_EVENT))) {
+    if(!eventlog) {
+        return false;
+    }
+
+    if(size < sizeof(TCG_EVENT)) {
         return false;
     }
 
