@@ -21,6 +21,7 @@ union tpm2_option_flags {
         uint8_t verbose :1;
         uint8_t quiet :1;
         uint8_t enable_errata :1;
+        uint8_t tcti_none :1;
     };
     uint8_t all;
 };
@@ -66,6 +67,7 @@ typedef bool (*tpm2_arg_handler)(int argc, char **argv);
  */
 #define TPM2_OPTIONS_NO_SAPI 0x1
 #define TPM2_OPTIONS_OPTIONAL_SAPI 0x2
+#define TPM2_OPTIONS_OPTIONAL_SAPI_AND_FAKE_TCTI 0x4
 
 struct tpm2_options {
     struct {
