@@ -20,7 +20,7 @@ static bool evaluate_populate_pcr_digests(TPML_PCR_SELECTION *pcr_selections,
     unsigned dgst_cnt = 0;
 
     //Iterating the number of pcr banks selected
-    UINT8 i;
+    UINT32 i;
     for (i = 0; i < pcr_selections->count; i++) {
 
         UINT8 total_indices_for_this_alg = 0;
@@ -80,7 +80,8 @@ static bool evaluate_populate_pcr_digests(TPML_PCR_SELECTION *pcr_selections,
 }
 
 tool_rc tpm2_policy_build_pcr(ESYS_CONTEXT *ectx, tpm2_session *policy_session,
-        const char *raw_pcrs_file, TPML_PCR_SELECTION *pcr_selections) {
+        const char *raw_pcrs_file, TPML_PCR_SELECTION *pcr_selections,
+        TPM2B_DIGEST *raw_pcr_digest) {
 
     TPML_DIGEST pcr_values = { .count = 0 };
 
@@ -89,6 +90,26 @@ tool_rc tpm2_policy_build_pcr(ESYS_CONTEXT *ectx, tpm2_session *policy_session,
         return tool_rc_general_error;
     }
 
+
+    TPM2B_DIGEST pcr_digest = TPM2B_TYPE_INIT(TPM2B_DIGEST, buffer);
+    TPMI_ALG_HASH auth_hash = tpm2_session_get_authhash(policy_session);
+    ESYS_TR handle = tpm2_session_get_handle(policy_session);
+
+    /*
+     * If digest of all PCRs is directly given, handle it here.
+     */
+    if (raw_pcr_digest &&
+    raw_pcr_digest->size != tpm2_alg_util_get_hash_size(auth_hash)) {
+        LOG_ERR("Specified PCR digest length not suitable with the policy session digest");
+        return tool_rc_general_error;
+    }
+    // Call the PolicyPCR command
+    if (raw_pcr_digest) {
+        return tpm2_policy_pcr(ectx, handle, ESYS_TR_NONE, ESYS_TR_NONE,
+            ESYS_TR_NONE, raw_pcr_digest, pcr_selections);
+    }
+
+
     bool result = evaluate_populate_pcr_digests(pcr_selections, raw_pcrs_file,
             &pcr_values);
     if (!result) {
@@ -139,9 +160,6 @@ tool_rc tpm2_policy_build_pcr(ESYS_CONTEXT *ectx, tpm2_session *policy_session,
     }
 
     // Calculate hashes
-    TPM2B_DIGEST pcr_digest = TPM2B_TYPE_INIT(TPM2B_DIGEST, buffer);
-    TPMI_ALG_HASH auth_hash = tpm2_session_get_authhash(policy_session);
-
     result = tpm2_openssl_hash_pcr_values(auth_hash, &pcr_values, &pcr_digest);
     if (!result) {
         LOG_ERR("Could not hash pcr values");
@@ -149,8 +167,6 @@ tool_rc tpm2_policy_build_pcr(ESYS_CONTEXT *ectx, tpm2_session *policy_session,
     }
 
     // Call the PolicyPCR command
-    ESYS_TR handle = tpm2_session_get_handle(policy_session);
-
     return tpm2_policy_pcr(ectx, handle, ESYS_TR_NONE, ESYS_TR_NONE,
             ESYS_TR_NONE, &pcr_digest, pcr_selections);
 }
@@ -163,7 +179,7 @@ tool_rc tpm2_policy_build_policyauthorize(ESYS_CONTEXT *ectx,
     bool result = true;
     TPM2B_DIGEST approved_policy = { .size = 0 };
     if (policy_digest_path) {
-        approved_policy.size = UINT16_MAX;
+        approved_policy.size = sizeof(TPMU_HA);
         result = files_load_bytes_from_path(policy_digest_path,
             approved_policy.buffer, &approved_policy.size);
     }
@@ -256,6 +272,14 @@ tool_rc tpm2_policy_build_policytemplate(ESYS_CONTEXT *ectx,
     return tpm2_policy_template(ectx, policy_session_handle, template_hash);
 }
 
+tool_rc tpm2_policy_build_policycphash(ESYS_CONTEXT *ectx,
+    tpm2_session *session, const TPM2B_DIGEST *cphash) {
+
+    ESYS_TR policy_session_handle = tpm2_session_get_handle(session);
+
+    return tpm2_policy_cphash(ectx, policy_session_handle, cphash);
+}
+
 tool_rc tpm2_policy_build_policyauthvalue(ESYS_CONTEXT *ectx,
         tpm2_session *session) {
 
@@ -269,7 +293,7 @@ tool_rc tpm2_policy_build_policysecret(ESYS_CONTEXT *ectx,
         tpm2_session *policy_session, tpm2_loaded_object *auth_entity_obj,
         INT32 expiration, TPMT_TK_AUTH **policy_ticket,
         TPM2B_TIMEOUT **timeout, bool is_nonce_tpm,
-        const char *policy_qualifier_data) {
+        const char *policy_qualifier_data, TPM2B_DIGEST *cp_hash) {
 
     /*
      * Qualifier data is optional. If not specified default to 0
@@ -297,7 +321,8 @@ tool_rc tpm2_policy_build_policysecret(ESYS_CONTEXT *ectx,
     }
 
     rc = tpm2_policy_secret(ectx, auth_entity_obj, policy_session_handle,
-        expiration, policy_ticket, timeout, nonce_tpm, &policy_qualifier);
+        expiration, policy_ticket, timeout, nonce_tpm, &policy_qualifier,
+        cp_hash);
 
 tpm2_policy_build_policysecret_out:
     Esys_Free(nonce_tpm);
@@ -366,7 +391,8 @@ tool_rc tpm2_policy_build_policysigned(ESYS_CONTEXT *ectx,
         tpm2_session *policy_session, tpm2_loaded_object *auth_entity_obj,
         TPMT_SIGNATURE *signature, INT32 expiration, TPM2B_TIMEOUT **timeout,
         TPMT_TK_AUTH **policy_ticket, const char *policy_qualifier_data,
-        bool is_nonce_tpm, const char *raw_data_path) {
+        bool is_nonce_tpm, const char *raw_data_path,
+        const char *cphash_path) {
 
     bool result = true;
 
@@ -385,6 +411,19 @@ tool_rc tpm2_policy_build_policysigned(ESYS_CONTEXT *ectx,
         }
     }
 
+    /*
+     * CpHashA (digest of command parameters for approved command) optional.
+     * If not specified default to NULL
+     */
+    TPM2B_DIGEST cphash = TPM2B_EMPTY_INIT;
+
+    if (cphash_path) {
+        bool result = files_load_digest(cphash_path, &cphash);
+        if (!result) {
+            return tool_rc_general_error;
+        }
+    }
+
     ESYS_TR policy_session_handle = tpm2_session_get_handle(policy_session);
 
     TPM2B_NONCE *nonce_tpm = NULL;
@@ -402,7 +441,7 @@ tool_rc tpm2_policy_build_policysigned(ESYS_CONTEXT *ectx,
      */
     if (raw_data_path) {
         uint16_t raw_data_len = (nonce_tpm ? nonce_tpm->size : 0) +
-            sizeof(INT32) + policy_qualifier.size;
+            sizeof(INT32) + cphash.size + policy_qualifier.size;
 
         uint8_t *raw_data = malloc(raw_data_len);
         if (!raw_data) {
@@ -410,17 +449,22 @@ tool_rc tpm2_policy_build_policysigned(ESYS_CONTEXT *ectx,
             rc = tool_rc_general_error;
             goto tpm2_policy_build_policysigned_out;
         }
-        //nonceTPM
+        /* nonceTPM */
         uint16_t offset = 0;
         if (nonce_tpm) {
             memcpy(raw_data, nonce_tpm->buffer, nonce_tpm->size);
             offset += nonce_tpm->size;
         }
-        //expiration
+        /* expiration */
         UINT32 endswap_data = tpm2_util_endian_swap_32(expiration);
         memcpy(raw_data + offset, (UINT8 *)&endswap_data, sizeof(INT32));
         offset += sizeof(INT32);
-        //policyRef
+        /* cpHash */
+        if (cphash_path) {
+            memcpy(raw_data + offset, cphash.buffer, cphash.size);
+            offset += cphash.size;
+        }
+        /* policyRef */
         memcpy(raw_data + offset, policy_qualifier.buffer,
             policy_qualifier.size);
 
@@ -438,7 +482,7 @@ tool_rc tpm2_policy_build_policysigned(ESYS_CONTEXT *ectx,
 
     rc = tpm2_policy_signed(ectx, auth_entity_obj, policy_session_handle,
         signature, expiration, timeout, policy_ticket, &policy_qualifier,
-        nonce_tpm);
+        nonce_tpm, &cphash);
 
 tpm2_policy_build_policysigned_out:
     Esys_Free(nonce_tpm);
@@ -588,6 +632,7 @@ bool tpm2_policy_parse_policy_list(char *str, TPML_DIGEST *policy_list) {
                 hash = tpm2_alg_util_from_optarg(subtoken,
                         tpm2_alg_util_flags_hash);
                 if (hash == TPM2_ALG_ERROR) {
+                    LOG_ERR("Invalid/ Unspecified policy digest algorithm.");
                     return false;
                 }
             }
