@@ -124,8 +124,7 @@ static tool_rc start_auth_session(tpm2_session *session) {
                     &session->input->nonce_caller : NULL;
 
     tool_rc rc = tpm2_start_auth_session(session->internal.ectx, d->key,
-            d->bind, ESYS_TR_NONE, ESYS_TR_NONE, ESYS_TR_NONE, nonce,
-            d->session_type, &d->symmetric, d->auth_hash,
+            d->bind, nonce, d->session_type, &d->symmetric, d->auth_hash,
             &session->output.session_handle);
     if (rc != tool_rc_success) {
         return rc;
@@ -290,13 +289,9 @@ tool_rc tpm2_session_restore(ESYS_CONTEXT *ctx, const char *path, bool is_final,
     s->internal.ectx = ctx;
     dup_path = NULL;
 
-    TPM2_HANDLE sapi_handle = 0;
     TPMA_SESSION attrs = 0;
 
     if (ctx) {
-        tool_rc tmp_rc = tpm2_util_esys_handle_to_sys_handle(ctx, handle,
-                &sapi_handle);
-        UNUSED(tmp_rc);
 
         /* hack this in here, should be done when starting the session */
         tmp_rc = tpm2_sess_get_attributes(ctx, handle, &attrs);
@@ -307,8 +302,7 @@ tool_rc tpm2_session_restore(ESYS_CONTEXT *ctx, const char *path, bool is_final,
 
     *session = s;
 
-    LOG_INFO("Restored session: ESYS_TR(0x%x) SAPI(0x%x) attrs(0x%x)", handle,
-            sapi_handle, attrs);
+    LOG_INFO("Restored session: ESYS_TR(0x%x) attrs(0x%x)", handle, attrs);
 
     rc = tool_rc_success;
 
@@ -331,13 +325,7 @@ tool_rc tpm2_session_get_noncetpm(ESYS_CONTEXT *ectx, tpm2_session *s,
 
 tool_rc tpm2_session_close(tpm2_session **s) {
 
-    tpm2_session *session = *s;
-
-    FILE *session_file = NULL;
-
-    tool_rc rc = tool_rc_general_error;
-
-    if (!session) {
+    if (!*s) {
         return tool_rc_success;
     }
 
@@ -346,29 +334,26 @@ tool_rc tpm2_session_close(tpm2_session **s) {
      *   - password sessions are implicit
      *   - hmac sessions live the life of the tool
      */
+    tool_rc rc = tool_rc_success;
+    tpm2_session *session = *s;
     if (session->output.session_handle == ESYS_TR_PASSWORD) {
-        rc = tool_rc_success;
+        goto out2;
+    }
+
+    const char *path = session->internal.path;
+    FILE *session_file = path ? fopen(path, "w+b") : NULL;
+    if (path && !session_file) {
+        LOG_ERR("Could not open path \"%s\", due to error: \"%s\"", path,
+                strerror(errno));
+        rc = tool_rc_general_error;
         goto out;
     }
 
-    bool flush = session->internal.is_final;
-    const char *path = session->internal.path;
-    if (path) {
-        session_file = fopen(path, "w+b");
-        if (!session_file) {
-            LOG_ERR("Could not open path \"%s\", due to error: \"%s\"", path,
-                    strerror(errno));
-            goto out;
-        }
-    } else {
-        flush = true;
-    }
-
+    bool flush = path ? session->internal.is_final : true;
     if (flush) {
-
         rc = tpm2_flush_context(session->internal.ectx,
                 session->output.session_handle);
-        /* done use rc to inidcate status */
+        /* done, use rc to indicate status */
         goto out;
     }
 
@@ -378,6 +363,7 @@ tool_rc tpm2_session_close(tpm2_session **s) {
     bool result = files_write_header(session_file, SESSION_VERSION);
     if (!result) {
         LOG_ERR("Could not write context file header");
+        rc = tool_rc_general_error;
         goto out;
     }
 
@@ -387,6 +373,7 @@ tool_rc tpm2_session_close(tpm2_session **s) {
             sizeof(session_type));
     if (!result) {
         LOG_ERR("Could not write session type");
+        rc = tool_rc_general_error;
         goto out;
     }
 
@@ -395,6 +382,7 @@ tool_rc tpm2_session_close(tpm2_session **s) {
     result = files_write_16(session_file, hash);
     if (!result) {
         LOG_ERR("Could not write auth hash");
+        rc = tool_rc_general_error;
         goto out;
     }
 
@@ -404,24 +392,19 @@ tool_rc tpm2_session_close(tpm2_session **s) {
      */
 
     ESYS_TR handle = tpm2_session_get_handle(session);
-
-    TPM2_HANDLE sapi_handle = 0;
-    tpm2_util_esys_handle_to_sys_handle(session->internal.ectx, handle,
-            &sapi_handle);
-
-    LOG_INFO("Saved session: ESYS_TR(0x%x) SAPI(0x%x)", handle, sapi_handle);
-
-    rc = files_save_tpm_context_to_file(session->internal.ectx,
-            tpm2_session_get_handle(session), session_file);
+    LOG_INFO("Saved session: ESYS_TR(0x%x)", handle);
+    rc = files_save_tpm_context_to_file(session->internal.ectx, handle,
+    session_file);
     if (rc != tool_rc_success) {
         LOG_ERR("Could not write session context");
+        /* done, free session resources and use rc to indicate status */
     }
 
 out:
     if (session_file) {
         fclose(session_file);
     }
-
+out2:
     tpm2_session_free(s);
 
     return rc;
