@@ -13,7 +13,9 @@
 #include "tpm2.h"
 #include "tpm2_alg_util.h"
 #include "tpm2_attr_util.h"
+#include "tpm2_convert.h"
 #include "tpm2_openssl.h"
+#include "tpm2_session.h"
 #include "tpm2_tool.h"
 #include "tpm2_util.h"
 
@@ -45,11 +47,7 @@ bool tpm2_util_concat_buffer(TPM2B_MAX_BUFFER *result, TPM2B *append) {
         return false;
     }
 
-    if ((result->size + append->size) < result->size) {
-        return false;
-    }
-
-    if ((result->size + append->size) > TPM2_MAX_DIGEST_BUFFER) {
+    if (((UINT32)result->size + append->size) > TPM2_MAX_DIGEST_BUFFER) {
         return false;
     }
 
@@ -579,13 +577,20 @@ bool tpm2_util_calc_unique(TPMI_ALG_HASH name_alg,
     memcpy(buf.buffer, seed->buffer, seed->size);
     memcpy(&buf.buffer[seed->size], key->buffer, key->size);
 
-    digester d = tpm2_openssl_halg_to_digester(name_alg);
-    if (!d) {
+    const EVP_MD *md = tpm2_openssl_md_from_tpmhalg(name_alg);
+    if (!md) {
+        LOG_ERR("Algorithm not supported: %x", name_alg);
         return false;
     }
 
-    unique_data->size = tpm2_alg_util_get_hash_size(name_alg);
-    d(buf.buffer, buf.size, unique_data->buffer);
+    unsigned int hash_size;
+    int rc = EVP_Digest(buf.buffer, buf.size, unique_data->buffer, &hash_size,
+                        md, NULL);
+    if (!rc) {
+        LOG_ERR("Hash calculation failed");
+        return false;
+    }
+    unique_data->size = hash_size;
 
     return true;
 }
@@ -944,7 +949,7 @@ bool tpm2_calq_qname(TPM2B_NAME *pqname,
     // QNB ≔ HB (QNA || NAMEB)
     bool result = false;
 
-    const EVP_MD *md = tpm2_openssl_halg_from_tpmhalg(halg);
+    const EVP_MD *md = tpm2_openssl_md_from_tpmhalg(halg);
 
     EVP_MD_CTX *mdctx = EVP_MD_CTX_create();
     if (!mdctx) {
@@ -993,3 +998,262 @@ out:
     EVP_MD_CTX_destroy(mdctx);
     return result;
 }
+
+bool tpm2_safe_read_from_stdin(int length, char *data) {
+    int rc;
+
+    char *buf = malloc(length);
+    char *read_data = malloc(length);
+
+    if (buf == fgets(buf, length, stdin)) {
+        rc = sscanf(buf, "%s", read_data);
+        if (rc != 1) {
+            free(buf);
+            free(read_data);
+            return false;
+        }
+    }
+    else {
+        free(buf);
+        free(read_data);
+        return false;
+    }
+
+    strcpy(data, read_data);
+    free(buf);
+    free(read_data);
+    return true;
+}
+
+bool tpm2_pem_encoded_key_to_fingerprint(const char *pem_encoded_key,
+    char *fingerprint) {
+
+    bool is_pemkey_len_valid = strlen(pem_encoded_key) > 1024 ? false : true;
+    if (!is_pemkey_len_valid) {
+        return false;
+    }
+
+    char str[1024] = "";
+    strcpy(str, pem_encoded_key);
+
+    /* walk through other tokens */
+    char base64[1024] = "";
+    char *token = strtok(str, "\n");
+    while ( token != NULL ) {
+        if (!strstr(token, "-----")) {
+            bool is_base64_overrun = (strlen(base64) + strlen(token)) > 1024 ?
+                true : false;
+            if (is_base64_overrun) {
+                return false;
+            }
+            strcat(base64, token);
+        }
+        token = strtok(NULL, "\n");
+    }
+
+    BYTE buffer[1024];
+    size_t buffer_length = 0;
+    int rc = tpm2_base64_decode(base64, buffer, &buffer_length);
+    if(!rc){
+        LOG_ERR("%s", "tpm2_base64_decode");
+        return false;
+    }
+
+    TPM2B_DIGEST digest;
+    rc = tpm2_openssl_hash_compute_data(TPM2_ALG_SHA256, buffer,
+        buffer_length, &digest);
+    if(!rc){
+        LOG_ERR("%s", "tpm2_openssl_hash_compute_data");
+        return false;
+    }
+
+    rc = tpm2_base64_encode(buffer, buffer_length, base64);
+    if(!rc){
+        LOG_ERR("%s", "tpm2_base64_decode");
+        return false;
+    }
+    strcpy(fingerprint, "SHA256:");
+    strcat(fingerprint, base64);
+
+    fingerprint[strlen(fingerprint)-1] = 0; // remove trailing \n
+
+    return true;
+}
+
+#define MAX_SESSION_CNT 3
+tool_rc tpm2_util_aux_sessions_setup(ESYS_CONTEXT *ectx, uint8_t session_cnt,
+    const char **session_path, ESYS_TR *session_handle,
+    tpm2_session **session) {
+
+    /*
+     * If no aux sessions were specified, simply return.
+     */
+    if (!session_cnt) {
+        return tool_rc_success;
+    }
+
+    if (session_cnt > MAX_SESSION_CNT) {
+        LOG_ERR("A max of 3 sessions allowed");
+        return tool_rc_general_error;
+    }
+
+    uint8_t session_idx = 0;
+    for (session_idx = 0; session_idx < (session_cnt); session_idx++) {
+        if (session_path[session_idx]) {
+                tool_rc rc = tpm2_session_restore(ectx,
+                    session_path[session_idx], false, &session[session_idx]);
+            if (rc != tool_rc_success) {
+                LOG_ERR("Could not restore aux-session #%s",
+                session_path[session_idx]);
+                return rc;
+            }
+            session_handle[session_idx] =
+                tpm2_session_get_handle(session[session_idx]);
+        }
+    }
+
+    return tool_rc_success;
+}
+
+static TPMI_ALG_HASH calc_phash_alg_from_phash_path(const char **phash_path) {
+
+    if (!*phash_path) {
+        return TPM2_ALG_ERROR;
+    }
+
+    /*
+     * Expecting single token, so tokenize just once.
+     */
+    char *str = malloc(strlen(*phash_path) + 1);
+    strcpy(str, *phash_path);
+    char *token = strtok(str, ":");
+
+    TPMI_ALG_HASH hashalg = tpm2_alg_util_from_optarg(
+        token, tpm2_alg_util_flags_hash);
+    /*
+     * Adjust the pHash path to skip the <halg>:
+     */
+    if (hashalg != TPM2_ALG_ERROR) {
+        *phash_path += strlen(token) + 1;
+    }
+
+    free(str);
+    return hashalg;
+}
+
+static TPMI_ALG_HASH tpm2_util_calc_phash_algorithm_from_session_types(
+    ESYS_CONTEXT *ectx, tpm2_session **sessions) {
+
+    TPMI_ALG_HASH rethash = TPM2_ALG_ERROR;
+
+    size_t session_idx = 0;
+    for (session_idx = 0; session_idx < MAX_SESSION_CNT; session_idx++) {
+        if(!sessions[session_idx]) {
+            continue;
+        }
+
+        /*
+         * Ignore password sessions
+         */
+        ESYS_TR session_handle = tpm2_session_get_handle(sessions[session_idx]);
+        if(session_handle == ESYS_TR_PASSWORD) {
+            continue;
+        }
+
+        /*
+         * Ignore trial sessions
+         */
+        TPM2_SE session_type = tpm2_session_get_type(sessions[session_idx]);
+        if (session_type != TPM2_SE_HMAC && session_type != TPM2_SE_POLICY) {
+            continue;
+        }
+
+        /*
+         * If this is an audit session, use that session halg.
+         * Note: Audit sessions are always HMAC type.
+         */
+        if (session_type == TPM2_SE_HMAC) {
+            TPMA_SESSION attrs = 0;
+            tool_rc tmp_rc = tpm2_sess_get_attributes(ectx, session_handle,
+                &attrs);
+            UNUSED(tmp_rc);
+
+            if (attrs & TPMA_SESSION_AUDIT) {
+                rethash = tpm2_session_get_authhash(sessions[session_idx]);
+                break;
+            }
+        }
+
+        /*
+         * If no other sessions remain, simply use (policy)sessions halg.
+         */
+        rethash = tpm2_session_get_authhash(sessions[session_idx]);
+    }
+
+    return rethash;
+}
+
+/*
+ * It should be noted that the auths aren't checked when calculating the pHash,
+ * instead the sessions are consumed to determine the pHash algorithm.
+ *
+ * 1. If phash_path is preceded with <halg>: use that as phash-halg return
+ * Otherwise
+ *
+ *  Consume session only if it is a policy-session or an hmac-session
+ *  1. If only hmac or policy session is specified, return that session's halg
+ *  2. If hmac-session with audit is specified, return that session's halg
+ *  3. If policy-session, then return policy-session's halg
+ *
+ * Otherwise
+ *  return SHA256
+ *
+ */
+TPMI_ALG_HASH tpm2_util_calculate_phash_algorithm(ESYS_CONTEXT *ectx,
+    const char **cphash_path, TPM2B_DIGEST *cp_hash, const char **rphash_path,
+    TPM2B_DIGEST *rp_hash, tpm2_session **sessions) {
+
+    /* <halg> specified in pHash path */
+    TPMI_ALG_HASH cphash_alg = cphash_path ? calc_phash_alg_from_phash_path(
+        cphash_path) : TPM2_ALG_ERROR;
+
+    TPMI_ALG_HASH rphash_alg = rphash_path ? calc_phash_alg_from_phash_path(
+        rphash_path) : TPM2_ALG_ERROR;
+    /*
+     * Default to cphash_alg if both are specified.
+     * This removes the conflict if cphash_alg and rphash_alg don't match.
+     * This also sets the cphash_alg if only rphash_alg is specified and vice
+     * versa.
+     */
+    TPMI_ALG_HASH phash_alg = cphash_alg != TPM2_ALG_ERROR ? cphash_alg :
+        (rphash_alg != TPM2_ALG_ERROR ? rphash_alg : TPM2_ALG_ERROR);
+
+    if (phash_alg != TPM2_ALG_ERROR) {
+        goto out;
+    }
+
+    /* <halg> determined from the sessions */
+    if (sessions) {
+        phash_alg = tpm2_util_calc_phash_algorithm_from_session_types(ectx,
+            sessions);
+    }
+
+out:
+    /* <halg> defaults to TPM2_ALG_SHA256 if cannot find from path or sessions */
+    if (phash_alg == TPM2_ALG_ERROR) {
+        phash_alg = TPM2_ALG_SHA256;
+    }
+
+    /*
+     * Side-effect: Set the size of the cp_hash and/or rp_hash
+     */
+    if (cphash_path && cp_hash) {
+        cp_hash->size = tpm2_alg_util_get_hash_size(phash_alg);
+    }
+
+    if (rphash_path && rp_hash) {
+        rp_hash->size = tpm2_alg_util_get_hash_size(phash_alg);
+    }
+
+    return phash_alg;
+}
