@@ -111,4 +111,25 @@ else
     true
 fi
 
+# Test bounded sessions work with bind entities with auth
+tpm2 createprimary -Q -C o -c prim.ctx -p primepass
+## Test with bounded and salted session
+tpm2 startauthsession -S session.ctx --hmac-session --tpmkey-context prim.ctx \
+--bind-context prim.ctx --bind-auth primepass
+tpm2 sessionconfig session.ctx --enable-encrypt --enable-decrypt
+tpm2 getrandom 8 -S session.ctx
+tpm2 flushcontext session.ctx
+## Test with bounded only session
+tpm2 startauthsession -S session.ctx --hmac-session \
+--bind-context prim.ctx  --bind-auth primepass
+tpm2 sessionconfig session.ctx --enable-encrypt --enable-decrypt
+tpm2 getrandom 8 -S session.ctx
+tpm2 flushcontext session.ctx
+## Test with salted only session
+tpm2 startauthsession -S session.ctx --hmac-session \
+--tpmkey-context prim.ctx
+tpm2 sessionconfig session.ctx --enable-encrypt --enable-decrypt
+tpm2 getrandom 8 -S session.ctx
+tpm2 flushcontext session.ctx
+
 exit 0
