@@ -23,7 +23,7 @@ cleanup "no-shutdown"
 
 generate_policycphash() {
     tpm2 startauthsession -S session.ctx -g sha256
-    tpm2 policycphash -S session.ctx -L policy.cphash --cphash cp.hash
+    tpm2 policycphash -S session.ctx -L policy.cphash --cphash-input cp.hash
     tpm2 flushcontext session.ctx
     rm session.ctx
 }
@@ -38,7 +38,7 @@ sign_and_verify_policycphash() {
 
 setup_authorized_policycphash() {
     tpm2 startauthsession -S session.ctx --policy-session -g sha256
-    tpm2 policycphash -S session.ctx --cphash cp.hash
+    tpm2 policycphash -S session.ctx --cphash-input cp.hash
     tpm2 policyauthorize -S session.ctx -i policy.cphash -n signing_key.name \
     -t verification.tkt
 }
@@ -46,12 +46,12 @@ setup_authorized_policycphash() {
 setup_owner_policy() {
     tpm2 setprimarypolicy -C o -L policy.cphash -g sha256
     tpm2 startauthsession -S session.ctx --policy-session -g sha256
-    tpm2 policycphash -S session.ctx --cphash cp.hash
+    tpm2 policycphash -S session.ctx --cphash-input cp.hash
 }
 
 start_policy_cphash() {
     tpm2 startauthsession -S session.ctx --policy-session -g sha256
-    tpm2 policycphash -S session.ctx --cphash cp.hash
+    tpm2 policycphash -S session.ctx --cphash-input cp.hash
 }
 
 create_authorized_policy() {
@@ -69,11 +69,23 @@ create_authorized_policy() {
 # Restrict the value that can be set through tpm2 nvsetbits.
 create_authorized_policy
 tpm2 nvdefine 1 -a "policywrite|authwrite|ownerread|nt=bits" -L authorized.policy
+NV_INDEX_NAME=$(tpm2 nvreadpublic | grep name | awk {'print $2'})
+#
+# Suppose TPM2_NVDefine was not run and the name was entirely created external
+# to the TPM. Note: This support still needs to be added.
+#
+# To emulate the absence of NV index while calculating the cpHash of NV_SetBits,
+# let's Undefine the NV index and define it later.
+#
+tpm2 nvundefine 1 -C o
+# (1) Calculate the cpHash
+tpm2 nvsetbits 1 -i 1 --cphash cp.hash -n $NV_INDEX_NAME --tcti=none
 ## Create policycphash
-tpm2 nvsetbits 1 -i 1 --cphash cp.hash
 generate_policycphash
 ## Sign and verify policycphash
 sign_and_verify_policycphash
+# (3) Define the NV index and write some NV index data
+tpm2 nvdefine 1 -a "policywrite|authwrite|ownerread|nt=bits" -L authorized.policy
 ## Satisfy policycphash and execute nvsetbits
 setup_authorized_policycphash
 tpm2 nvsetbits 1 -i 1 -P "session:session.ctx"
@@ -93,9 +105,24 @@ tpm2 nvundefine 1
 # Test tpm2 nvextend
 create_authorized_policy
 tpm2 nvdefine 1 -a "nt=extend|ownerread|policywrite" -L authorized.policy
-echo "foo" | tpm2 nvextend -i- 1 --cphash cp.hash
+NV_INDEX_NAME=$(tpm2 nvreadpublic | grep name | awk {'print $2'})
+#
+# Suppose TPM2_NVDefine was not run and the name was entirely created external
+# to the TPM. Note: This support still needs to be added.
+#
+# To emulate the absence of NV index while calculating the cpHash of NV_SetBits,
+# let's Undefine the NV index and define it later.
+#
+tpm2 nvundefine 1 -C o
+# (1) Calculate the cpHash
+echo "foo" | tpm2 nvextend -i- 1 --cphash cp.hash -n $NV_INDEX_NAME --tcti=none
+## Create policycphash
 generate_policycphash
+## Sign and verify policycphash
 sign_and_verify_policycphash
+# (3) Define the NV index
+tpm2 nvdefine 1 -a "nt=extend|ownerread|policywrite" -L authorized.policy
+## Satisfy policycphash and execute nvsetbits
 setup_authorized_policycphash
 echo "foo" | tpm2 nvextend -i- 1 -P "session:session.ctx"
 tpm2 flushcontext session.ctx
@@ -124,24 +151,40 @@ tpm2 nvundefine 1
 
 # Test tpm2 nvread
 create_authorized_policy
-tpm2 nvdefine 1 -s 8 -a "ownerwrite|policyread" -L authorized.policy
-echo "foo" | tpm2 nvwrite 1 -i- -C o
-tpm2 nvread 1 -s 8 --cphash cp.hash
+tpm2 nvdefine 1 -a "ownerwrite|policyread" -L authorized.policy
+echo "some nv data" | tpm2 nvwrite 1 -i- -C o
+NV_INDEX_NAME=$(tpm2 nvreadpublic | grep name | awk {'print $2'})
+#
+# Suppose TPM2_NVDefine was not run and the name was entirely created external
+# to the TPM. Note: This support still needs to be added.
+#
+# To emulate the absence of NV index while calculating the cpHash of NVRead,
+# let's Undefine the NV index and define it later.
+#
+tpm2 nvundefine 1 -C o
+# (1) Calculate the cpHash
+tpm2 nvread 1 -s 4 --cphash cp.hash -n $NV_INDEX_NAME --tcti=none
+# (2) Calculate the policycphash
 generate_policycphash
 sign_and_verify_policycphash
+# (3) Define the NV index and write some NV index data
+tpm2 nvdefine 1 -a "ownerwrite|policyread" -L authorized.policy
+echo "some nv data" | tpm2 nvwrite 1 -i- -C o
+# (4)
 setup_authorized_policycphash
-tpm2 nvread 1 -s 8 -P "session:session.ctx" | xxd -p
+tpm2 nvread 1 -s 4 -P "session:session.ctx"
+tpm2 flushcontext session.ctx
 ## test the failing scenario
 setup_authorized_policycphash
 trap - ERR
-tpm2 nvread 1 -s 7 --offset 1 -P "session:session.ctx"
+tpm2 nvread 1 -s 8 --offset 1 -P "session:session.ctx"
 if [ $? == 0 ];then
   echo "ERROR: nvread must fail!"
   exit 1
 fi
 trap onerror ERR
 tpm2 flushcontext session.ctx
-tpm2 nvundefine 1
+tpm2 nvundefine 1 -C o
 
 # Test tpm2 nvreadlock
 create_authorized_policy
@@ -320,7 +363,7 @@ fi
 trap onerror ERR
 tpm2 flushcontext session.ctx
 
-# Test tpm2 activatecredential
+# Test tpm2 activatecredential (using legacy option --cphash)
 create_authorized_policy
 tpm2 createprimary -C o -c prim.ctx -G rsa
 tpm2 readpublic -c prim.ctx -o prim.pub
@@ -394,12 +437,12 @@ dd if=/dev/urandom of=sym_key_in.bin bs=1 count=16 status=none
 tpm2 duplicate -C new_parent.ctx -c duplicable_key.ctx -G aes \
 -i sym_key_in.bin -r dupprv.bin -s dupseed.dat --cphash cp.hash
 tpm2 startauthsession -S session.ctx -g sha256
-tpm2 policycphash -S session.ctx --cphash cp.hash
+tpm2 policycphash -S session.ctx --cphash-input cp.hash
 tpm2 policycommandcode -S session.ctx -L policy.cphash TPM2_CC_Duplicate
 tpm2 flushcontext session.ctx
 sign_and_verify_policycphash
 tpm2 startauthsession --policy-session -S session.ctx -g sha256
-tpm2 policycphash -S session.ctx --cphash cp.hash
+tpm2 policycphash -S session.ctx --cphash-input cp.hash
 tpm2 policycommandcode -S session.ctx TPM2_CC_Duplicate
 tpm2 policyauthorize -S session.ctx -i policy.cphash -n signing_key.name \
 -t verification.tkt
@@ -409,7 +452,7 @@ tpm2 flushcontext session.ctx
 ## attempt failing scenario
 dd if=/dev/urandom of=sym_key_in.bin bs=1 count=16 status=none
 tpm2 startauthsession --policy-session -S session.ctx -g sha256
-tpm2 policycphash -S session.ctx --cphash cp.hash
+tpm2 policycphash -S session.ctx --cphash-input cp.hash
 tpm2 policycommandcode -S session.ctx TPM2_CC_Duplicate
 tpm2 policyauthorize -S session.ctx -i policy.cphash -n signing_key.name \
 -t verification.tkt
@@ -480,12 +523,12 @@ tpm2 create -C prim.ctx -c key.ctx -G rsa  -u key.pub -r key.priv \
 -L authorized.policy -P primarypass
 tpm2 certify -c prim.ctx -C key.ctx -g sha256 --cphash cp.hash
 tpm2 startauthsession -S session.ctx -g sha256
-tpm2 policycphash -S session.ctx --cphash cp.hash
+tpm2 policycphash -S session.ctx --cphash-input cp.hash
 tpm2 policycommandcode -S session.ctx -L policy.cphash TPM2_CC_Certify
 tpm2 flushcontext session.ctx
 sign_and_verify_policycphash
 tpm2 startauthsession --policy-session -S session.ctx -g sha256
-tpm2 policycphash -S session.ctx --cphash cp.hash
+tpm2 policycphash -S session.ctx --cphash-input cp.hash
 tpm2 policycommandcode -S session.ctx TPM2_CC_Certify
 tpm2 policyauthorize -S session.ctx -i policy.cphash -n signing_key.name \
 -t verification.tkt
@@ -563,7 +606,7 @@ tpm2 hierarchycontrol -C p shEnable clear --cphash cp.hash
 generate_policycphash
 tpm2 setprimarypolicy -C p -L policy.cphash -g sha256
 tpm2 startauthsession -S session.ctx --policy-session -g sha256
-tpm2 policycphash -S session.ctx --cphash cp.hash
+tpm2 policycphash -S session.ctx --cphash-input cp.hash
 tpm2 hierarchycontrol -C p shEnable clear -P "session:session.ctx"
 tpm2 flushcontext session.ctx
 
@@ -586,7 +629,7 @@ tpm2 clear -c l --cphash cp.hash
 generate_policycphash
 tpm2 setprimarypolicy -C l -L policy.cphash -g sha256
 tpm2 startauthsession -S session.ctx --policy-session -g sha256
-tpm2 policycphash -S session.ctx --cphash cp.hash
+tpm2 policycphash -S session.ctx --cphash-input cp.hash
 tpm2 clear -c l "session:session.ctx"
 tpm2 flushcontext session.ctx
 
@@ -596,7 +639,7 @@ tpm2 clearcontrol -C l s --cphash cp.hash
 generate_policycphash
 tpm2 setprimarypolicy -C l -L policy.cphash -g sha256
 tpm2 startauthsession -S session.ctx --policy-session -g sha256
-tpm2 policycphash -S session.ctx --cphash cp.hash
+tpm2 policycphash -S session.ctx --cphash-input cp.hash
 tpm2 clearcontrol -C l s -P "session:session.ctx"
 tpm2 flushcontext session.ctx
 
@@ -607,7 +650,7 @@ tpm2 dictionarylockout -s -n 5 -t 6 -l 7 --cphash cp.hash
 generate_policycphash
 tpm2 setprimarypolicy -C l -L policy.cphash -g sha256
 tpm2 startauthsession -S session.ctx --policy-session -g sha256
-tpm2 policycphash -S session.ctx --cphash cp.hash
+tpm2 policycphash -S session.ctx --cphash-input cp.hash
 tpm2 dictionarylockout -s -n 5 -t 6 -l 7 --cphash cp.hash -p "session:session.ctx"
 tpm2 flushcontext session.ctx
 
@@ -676,4 +719,20 @@ setup_authorized_policycphash
 tpm2 hmac -c key.ctx plain.txt -o hmac.bin -p "session:session.ctx"
 tpm2 flushcontext session.ctx
 
+# Test if specifying the <halg>: in the cphash path has effect on cphash alg
+TPM2_ALG_SHA256_SZ=34
+TPM2_ALG_SHA1_SZ=22
+
+## Default must be sha256
+expected_hash_len=$TPM2_ALG_SHA256_SZ
+tpm2 getrandom 8 --cphash cp.hash
+file_size=`ls -l cp.hash | awk {'print $5'}`
+test $file_size -eq $expected_hash_len
+
+## Check if a specific hash can be enforced with <halg>:
+expected_hash_len=$TPM2_ALG_SHA1_SZ
+tpm2 getrandom 8 --cphash sha1:cp.hash
+file_size=`ls -l cp.hash | awk {'print $5'}`
+test $file_size -eq $expected_hash_len
+
 exit 0
