@@ -130,9 +130,12 @@ static tool_rc set_ek_template(ESYS_CONTEXT *ectx, TPM2B_PUBLIC *input_public) {
     UINT8* nonce = NULL;
 
     // Read EK template
-    UINT16 template_size;
+    UINT16 template_size = 0;
+    TPM2B_DIGEST cp_hash = { 0 };
+    TPM2B_DIGEST rp_hash = { 0 };
     tool_rc rc = tpm2_util_nv_read(ectx, template_nv_index, 0, 0,
-        &ctx.auth_owner_hierarchy.object, &template, &template_size, NULL);
+        &ctx.auth_owner_hierarchy.object, &template, &template_size, &cp_hash,
+        &rp_hash, TPM2_ALG_SHA256, 0, ESYS_TR_NONE, ESYS_TR_NONE);
     if (rc != tool_rc_success) {
         goto out;
     }
@@ -146,9 +149,10 @@ static tool_rc set_ek_template(ESYS_CONTEXT *ectx, TPM2B_PUBLIC *input_public) {
     }
 
     // Read EK nonce
-    UINT16 nonce_size;
+    UINT16 nonce_size = 0;
     rc = tpm2_util_nv_read(ectx, nonce_nv_index, 0, 0,
-        &ctx.auth_owner_hierarchy.object, &nonce, &nonce_size, NULL);
+        &ctx.auth_owner_hierarchy.object, &nonce, &nonce_size, &cp_hash,
+        &rp_hash, TPM2_ALG_SHA256, 0, ESYS_TR_NONE, ESYS_TR_NONE);
     if (rc != tool_rc_success) {
         goto out;
     }
