@@ -54,19 +54,16 @@ struct tpm_import_ctx {
     char *name_alg;
     char *attrs; /* The attributes to use */
     char *key_auth_str;
-    char *auth_key_file; /* an optional auth string for the input key file for OSSL */
+    char *passin; /* an optional auth string for the input key file for OSSL */
     char *input_seed_file;
     char *input_enc_key_file;
     char *policy;
     bool import_tpm; /* Any param that is exclusively used by import tpm object sets this flag */
-    TPMI_ALG_PUBLIC key_type;
+    char *object_alg;
     char *cp_hash_path;
 };
 
-static tpm_import_ctx ctx = {
-    .key_type = TPM2_ALG_ERROR,
-    .input_key_file = NULL,
-};
+static tpm_import_ctx ctx;
 
 static tool_rc readpublic(ESYS_CONTEXT *ectx, ESYS_TR handle,
         TPM2B_PUBLIC **public) {
@@ -74,47 +71,7 @@ static tool_rc readpublic(ESYS_CONTEXT *ectx, ESYS_TR handle,
     return tpm2_readpublic(ectx, handle, public, NULL, NULL);
 }
 
-static bool create_name(TPM2B_PUBLIC *public, TPM2B_NAME *pubname) {
-
-    /*
-     * A TPM2B_NAME is the name of the algorithm, followed by the hash.
-     * Calculate the name by:
-     * 1. Marshaling the name algorithm
-     * 2. Marshaling the TPMT_PUBLIC past the name algorithm from step 1.
-     * 3. Hash the TPMT_PUBLIC portion in marshaled data.
-     */
-
-    TPMI_ALG_HASH name_alg = public->publicArea.nameAlg;
-
-    // Step 1 - set beginning of name to hash alg
-    size_t hash_offset = 0;
-    Tss2_MU_UINT16_Marshal(name_alg, pubname->name, pubname->size,
-            &hash_offset);
-
-    // Step 2 - marshal TPMTP
-    TPMT_PUBLIC marshaled_tpmt;
-    size_t tpmt_marshalled_size = 0;
-    Tss2_MU_TPMT_PUBLIC_Marshal(&public->publicArea,
-            (uint8_t *) &marshaled_tpmt, sizeof(public->publicArea),
-            &tpmt_marshalled_size);
-
-    // Step 3 - Hash the data into name just past the alg type.
-    digester d = tpm2_openssl_halg_to_digester(name_alg);
-    if (!d) {
-        return false;
-    }
-
-    d((const unsigned char *) &marshaled_tpmt, tpmt_marshalled_size,
-            pubname->name + 2);
-
-    //Set the name size, UINT16 followed by HASH
-    UINT16 hash_size = tpm2_alg_util_get_hash_size(name_alg);
-    pubname->size = hash_size + 2;
-
-    return true;
-}
-
-static void create_import_key_private_data(TPM2B_PRIVATE *private,
+static bool create_import_key_private_data(TPM2B_PRIVATE *private,
         TPMI_ALG_HASH parent_name_alg,
         TPM2B_MAX_BUFFER *encrypted_duplicate_sensitive,
         TPM2B_DIGEST *outer_hmac) {
@@ -122,16 +79,25 @@ static void create_import_key_private_data(TPM2B_PRIVATE *private,
     //UINT16 hash_size = tpm2_alg_util_get_hash_size(ctx.name_alg);
     UINT16 parent_hash_size = tpm2_alg_util_get_hash_size(parent_name_alg);
 
-private->size = sizeof(uint16_t) + parent_hash_size
+    private->size = sizeof(parent_hash_size) + parent_hash_size
             + encrypted_duplicate_sensitive->size;
+
     size_t hmac_size_offset = 0;
-    Tss2_MU_UINT16_Marshal(parent_hash_size, private->buffer, sizeof(uint16_t),
-            &hmac_size_offset);
+    TSS2_RC rval = Tss2_MU_UINT16_Marshal(parent_hash_size, private->buffer,
+            sizeof(parent_hash_size), &hmac_size_offset);
+    if (rval != TPM2_RC_SUCCESS)
+    {
+        LOG_ERR("Error serializing parent hash size");
+        return false;
+    }
+
     memcpy(private->buffer + hmac_size_offset, outer_hmac->buffer,
             parent_hash_size);
     memcpy(private->buffer + hmac_size_offset + parent_hash_size,
             encrypted_duplicate_sensitive->buffer,
             encrypted_duplicate_sensitive->size);
+
+    return true;
 }
 
 static tool_rc key_import(ESYS_CONTEXT *ectx, TPM2B_PUBLIC *parent_pub,
@@ -149,27 +115,43 @@ static tool_rc key_import(ESYS_CONTEXT *ectx, TPM2B_PUBLIC *parent_pub,
     TPM2B_DATA enc_sensitive_key = {
         .size = parent_pub->publicArea.parameters.rsaDetail.symmetric.keyBits.sym / 8
     };
-    memset(enc_sensitive_key.buffer, 0xFF, enc_sensitive_key.size);
+
+    if(enc_sensitive_key.size < 16) {
+        LOG_ERR("Calculated wrapping keysize is less than 16 bytes, got: %u", enc_sensitive_key.size);
+        return tool_rc_general_error;
+    }
+
+    int ossl_rc = RAND_bytes(enc_sensitive_key.buffer, enc_sensitive_key.size);
+    if (ossl_rc != 1) {
+        LOG_ERR("RAND_bytes failed: %s", ERR_error_string(ERR_get_error(), NULL));
+        return tool_rc_general_error;
+    }
 
     /*
      * Calculate the object name.
      */
     TPM2B_NAME pubname = TPM2B_TYPE_INIT(TPM2B_NAME, name);
-    bool res = create_name(pubkey, &pubname);
+    bool res = tpm2_identity_create_name(pubkey, &pubname);
     if (!res) {
-        return false;
+        return tool_rc_general_error;
     }
 
     TPM2B_MAX_BUFFER hmac_key;
     TPM2B_MAX_BUFFER enc_key;
-    tpm2_identity_util_calc_outer_integrity_hmac_key_and_dupsensitive_enc_key(
+    res = tpm2_identity_util_calc_outer_integrity_hmac_key_and_dupsensitive_enc_key(
             parent_pub, &pubname, seed, &hmac_key, &enc_key);
+    if (!res) {
+        return tool_rc_general_error;
+    }
 
     TPM2B_MAX_BUFFER encrypted_inner_integrity = TPM2B_EMPTY_INIT;
-    tpm2_identity_util_calculate_inner_integrity(name_alg, privkey, &pubname,
+    res = tpm2_identity_util_calculate_inner_integrity(name_alg, privkey, &pubname,
             &enc_sensitive_key,
             &parent_pub->publicArea.parameters.rsaDetail.symmetric,
             &encrypted_inner_integrity);
+    if (!res) {
+        return tool_rc_general_error;
+    }
 
     TPM2B_DIGEST outer_hmac = TPM2B_EMPTY_INIT;
     TPM2B_MAX_BUFFER encrypted_duplicate_sensitive = TPM2B_EMPTY_INIT;
@@ -179,8 +161,11 @@ static tool_rc key_import(ESYS_CONTEXT *ectx, TPM2B_PUBLIC *parent_pub,
             &encrypted_duplicate_sensitive, &outer_hmac);
 
     TPM2B_PRIVATE private = TPM2B_EMPTY_INIT;
-    create_import_key_private_data(&private, parent_pub->publicArea.nameAlg,
+    res = create_import_key_private_data(&private, parent_pub->publicArea.nameAlg,
             &encrypted_duplicate_sensitive, &outer_hmac);
+    if (!res) {
+        return tool_rc_general_error;
+    }
 
     TPMT_SYM_DEF_OBJECT *sym_alg =
             &parent_pub->publicArea.parameters.rsaDetail.symmetric;
@@ -214,12 +199,7 @@ static bool on_option(char key, char *value) {
         ctx.key_auth_str = value;
         break;
     case 'G':
-        ctx.key_type = tpm2_alg_util_from_optarg(value,
-                tpm2_alg_util_flags_asymmetric | tpm2_alg_util_flags_symmetric);
-        if (ctx.key_type == TPM2_ALG_ERROR) {
-            LOG_ERR("Unsupported key type");
-            return false;
-        }
+        ctx.object_alg = value;
         return true;
     case 'i':
         ctx.input_key_file = value;
@@ -254,7 +234,7 @@ static bool on_option(char key, char *value) {
         ctx.policy = value;
         break;
     case 0:
-        ctx.auth_key_file = value;
+        ctx.passin = value;
         break;
     case 1:
         ctx.cp_hash_path = value;
@@ -278,7 +258,7 @@ static bool tpm2_tool_onstart(tpm2_options **opts) {
       { "parent-public",      required_argument, NULL, 'U'},
       { "private",            required_argument, NULL, 'r'},
       { "public",             required_argument, NULL, 'u'},
-      { "attributes",  required_argument, NULL, 'a'},
+      { "attributes",         required_argument, NULL, 'a'},
       { "hash-algorithm",     required_argument, NULL, 'g'},
       { "seed",               required_argument, NULL, 's'},
       { "policy",             required_argument, NULL, 'L'},
@@ -310,18 +290,16 @@ static tool_rc check_options(void) {
             rc = tool_rc_option_error;
         }
 
-        /* If a key file is specified we choose aes else null
-         for symmetricAlgdefinition */
-        if (!ctx.input_enc_key_file) {
-            ctx.key_type = TPM2_ALG_NULL;
-        } else {
-            ctx.key_type = TPM2_ALG_AES;
+        if (ctx.key_auth_str) {
+            LOG_ERR("Cannot specify key password when importing a TPM key.\n"
+                "use tpm2_changeauth after import");
+            rc = tool_rc_option_error;
         }
 
     } else { /* Openssl specific option(s) */
 
-        if (!ctx.key_type) {
-            LOG_ERR("Expected key type to be specified via \"-G\","
+        if (!ctx.object_alg) {
+            LOG_ERR("Expected object type to be specified via \"-G\","
                     " missing option.");
             rc = tool_rc_option_error;
         }
@@ -360,16 +338,42 @@ static tool_rc check_options(void) {
     return rc;
 }
 
-static tool_rc openssl_import(ESYS_CONTEXT *ectx) {
+static void setup_default_attrs(TPMA_OBJECT *attrs, bool has_policy, bool has_auth) {
 
-    bool free_ppub = false;
-    tool_rc rc = tool_rc_general_error;
+    /* Handle Default Setup */
+    *attrs = DEFAULT_CREATE_ATTRS;
+
+    /* imported objects arn't created inside of the TPM so this gets turned down */
+    *attrs &= ~TPMA_OBJECT_SENSITIVEDATAORIGIN;
+    *attrs &= ~TPMA_OBJECT_FIXEDTPM;
+    *attrs &= ~TPMA_OBJECT_FIXEDPARENT;
+
+    /* The default for a keyedhash object with no scheme is just for sealing */
+    if (!strcmp("keyedhash", ctx.object_alg)) {
+        *attrs &= ~TPMA_OBJECT_SIGN_ENCRYPT;
+        *attrs &= ~TPMA_OBJECT_DECRYPT;
+    } else if (!strncmp("hmac", ctx.object_alg, 4)) {
+        *attrs &= ~TPMA_OBJECT_DECRYPT;
+    }
+
+    /*
+     * IMPORTANT: if the object we're creating has a policy and NO authvalue, turn off userwith auth
+     * so empty passwords don't work on the object.
+     */
+    if (has_policy && !has_auth) {
+        *attrs &= ~TPMA_OBJECT_USERWITHAUTH;
+    }
+}
+
+static tool_rc openssl_import(ESYS_CONTEXT *ectx) {
 
     /*
      * Load the parent public file, or read it from the TPM if not specified.
      * We need this information for encrypting the protection seed.
      */
+    bool free_ppub = false;
     tool_rc tmp_rc;
+    tool_rc rc = tool_rc_general_error;
     TPM2B_PUBLIC ppub = TPM2B_EMPTY_INIT;
     TPM2B_PUBLIC *parent_pub = NULL;
 
@@ -388,123 +392,52 @@ static tool_rc openssl_import(ESYS_CONTEXT *ectx) {
         return tmp_rc;
     }
 
-    TPMA_OBJECT attrs = TPMA_OBJECT_DECRYPT | TPMA_OBJECT_SIGN_ENCRYPT;
-
-    TPM2B_PUBLIC public = {
-        .size = 0,
-        .publicArea = {
-            .nameAlg = TPM2_ALG_SHA256,
-        },
-    };
-
-    if (ctx.policy) {
-        public.publicArea.authPolicy.size = sizeof(public.publicArea.authPolicy.buffer);
-        result = files_load_bytes_from_path(ctx.policy,
-            public.publicArea.authPolicy.buffer,
-                &public.publicArea.authPolicy.size);
-        if (!result) {
-            return tool_rc_general_error;
-        }
-    } else {
-        attrs |= TPMA_OBJECT_USERWITHAUTH;
-    }
-
     TPM2B_SENSITIVE private = TPM2B_EMPTY_INIT;
-
-    if (ctx.key_auth_str) {
-        tpm2_session *tmp;
-        tmp_rc = tpm2_auth_util_from_optarg(NULL, ctx.key_auth_str, &tmp, true);
-        if (tmp_rc != tool_rc_success) {
-            LOG_ERR("Invalid key authorization");
-            return tmp_rc;
-        }
-
-        const TPM2B_AUTH *auth = tpm2_session_get_auth_value(tmp);
-        private.sensitiveArea.authValue = *auth;
-
-        tpm2_session_close(&tmp);
-    }
-
-    /*
-     * Set the object attributes if specified, overwriting the defaults, but hooking the errata
-     * fixups.
-     */
-    if (ctx.attrs) {
-        TPMA_OBJECT *obj_attrs = &public.publicArea.objectAttributes;
-        result = tpm2_attr_util_obj_from_optarg(ctx.attrs, obj_attrs);
-        if (!result) {
-            LOG_ERR("Invalid object attribute, got\"%s\"", ctx.attrs);
-            return tool_rc_general_error;
-        }
-
-        tpm2_errata_fixup(SPEC_116_ERRATA_2_7,
-                &public.publicArea.objectAttributes);
-    } else {
-        public.publicArea.objectAttributes = attrs;
-    }
-
-    if (ctx.name_alg) {
-        TPMI_ALG_HASH alg = tpm2_alg_util_from_optarg(ctx.name_alg,
-                tpm2_alg_util_flags_hash);
-        if (alg == TPM2_ALG_ERROR) {
-            LOG_ERR("Invalid name hashing algorithm, got\"%s\"", ctx.name_alg);
-            return tool_rc_general_error;
-        }
-        public.publicArea.nameAlg = alg;
-    } else {
-        /*
-         * use the parent name algorithm if not specified
-         */
-    public.publicArea.nameAlg = parent_pub->publicArea.nameAlg;
-    }
-
-    /*
-     * The TPM Requires that the name algorithm for the child be less than the name
-     * algorithm of the parent when the parent's scheme is NULL.
-     *
-     * This check can be seen in the simulator at:
-     *   - File: CryptUtil.c
-     *   - Func: CryptSecretDecrypt()
-     *   - Line: 2019
-     *   - Decription: Limits the size of the hash algorithm to less then the parent's name-alg when scheme is NULL.
-     */
-    UINT16 hash_size = tpm2_alg_util_get_hash_size(public.publicArea.nameAlg);
-    UINT16 parent_hash_size = tpm2_alg_util_get_hash_size(
-            parent_pub->publicArea.nameAlg);
-    if (hash_size > parent_hash_size) {
-        LOG_WARN("Hash selected is larger then parent hash size, coercing to "
-                 "parent hash algorithm: %s",
-                tpm2_alg_util_algtostr(parent_pub->publicArea.nameAlg,
-                        tpm2_alg_util_flags_hash));
-    public.publicArea.nameAlg = parent_pub->publicArea.nameAlg;
-    }
-
-    /*
-     * Generate and encrypt seed
-     */
-    TPM2B_DIGEST *seed = &private.sensitiveArea.seedValue;
+    TPM2B_PUBLIC public = TPM2B_EMPTY_INIT;
     TPM2B_ENCRYPTED_SECRET encrypted_seed = TPM2B_EMPTY_INIT;
-    unsigned char label[10] = { 'D', 'U', 'P', 'L', 'I', 'C', 'A', 'T', 'E', 0 };
-    bool res;
-    res = tpm2_identity_util_share_secret_with_public_key(seed, parent_pub,
-            label, 10, &encrypted_seed);
-    if (!res) {
-        LOG_ERR("Failed Seed Encryption\n");
-        goto out;
+
+    /*
+     * start with the tools default set and turn off the ones that don't make sense
+     * If the user specified their own values, tpm2_alg_util_public_init will use that,
+     * so this is just the default case.
+     * */
+    TPMA_OBJECT attrs = 0;
+    if (!ctx.attrs) {
+        setup_default_attrs(&attrs, !!ctx.policy, !!ctx.key_auth_str);
     }
 
     /*
-     * Populate all the private and public data fields we can based on the key type and the PEM files read in.
+     * Backwards Compat: the tool sets name-alg by default to the parent name alg if not specified
+     * but the tpm2_alg_util_public_init defaults to sha256. Specify the alg if not specified.
      */
-    tpm2_openssl_load_rc status = tpm2_openssl_load_private(ctx.input_key_file,
-            ctx.auth_key_file, ctx.key_type, &public, &private);
-    if (status == lprc_error) {
+    if (!ctx.name_alg) {
+        ctx.name_alg = (char *)tpm2_alg_util_algtostr(parent_pub->publicArea.nameAlg,
+                tpm2_alg_util_flags_hash);
+        if (!ctx.name_alg) {
+            LOG_ERR("Invalid parent name algorithm, got 0x%x",
+                    parent_pub->publicArea.nameAlg);
+            goto out;
+        }
+    }
+
+    TPM2B_PUBLIC template = { 0 };
+    rc = tpm2_alg_util_public_init(ctx.object_alg, ctx.name_alg,
+        ctx.attrs, ctx.policy, attrs, &template);
+    if (rc != tool_rc_success) {
         goto out;
     }
 
-    if (!tpm2_openssl_did_load_public(status)) {
-        LOG_ERR("Did not find public key information in file: \"%s\"",
-                ctx.input_key_file);
+    result = tpm2_openssl_import_keys(
+        parent_pub,
+        &encrypted_seed,
+        ctx.key_auth_str,
+        ctx.input_key_file,
+        ctx.passin,
+        &template,
+        &private,
+        &public
+    );
+    if (!result) {
         goto out;
     }
 
@@ -519,13 +452,13 @@ static tool_rc openssl_import(ESYS_CONTEXT *ectx) {
     /*
      * Save the public and imported_private structure to disk
      */
-    res = files_save_public(&public, ctx.public_key_file);
-    if (!res) {
+    result = files_save_public(&public, ctx.public_key_file);
+    if (!result) {
         goto keyout;
     }
 
-    res = files_save_private(imported_private, ctx.private_key_file);
-    if (!res) {
+    result = files_save_private(imported_private, ctx.private_key_file);
+    if (!result) {
         goto keyout;
     }
 
@@ -537,53 +470,31 @@ static tool_rc openssl_import(ESYS_CONTEXT *ectx) {
     rc = tool_rc_success;
 
 keyout:
-    free(imported_private);
+    Esys_Free(imported_private);
 out:
     if (free_ppub) {
-        free(parent_pub);
+        Esys_Free(parent_pub);
     }
 
     return rc;
 }
 
-static bool set_key_algorithm(TPMI_ALG_PUBLIC alg, TPMT_SYM_DEF_OBJECT * obj) {
-    bool result = true;
-    switch (alg) {
-    case TPM2_ALG_AES:
-        obj->algorithm = TPM2_ALG_AES;
-        obj->keyBits.aes = 128;
-        obj->mode.aes = TPM2_ALG_CFB;
-        break;
-    case TPM2_ALG_NULL:
-        obj->algorithm = TPM2_ALG_NULL;
-        break;
-    default:
-        LOG_ERR("The algorithm type input(0x%x) is not supported!", alg);
-        result = false;
-        break;
-    }
-    return result;
-}
-
 static tool_rc tpm_import(ESYS_CONTEXT *ectx) {
 
+    tool_rc rc = tool_rc_general_error;
+
     TPM2B_DATA enc_key = TPM2B_EMPTY_INIT;
     TPM2B_PUBLIC public = TPM2B_EMPTY_INIT;
     TPM2B_PRIVATE duplicate;
     TPM2B_ENCRYPTED_SECRET encrypted_seed;
     TPM2B_PRIVATE *imported_private = NULL;
-    TPMT_SYM_DEF_OBJECT sym_alg;
 
-    tool_rc rc;
-    bool result = set_key_algorithm(ctx.key_type, &sym_alg);
-    if (!result) {
-        return tool_rc_general_error;
-    }
+    TPMT_SYM_DEF_OBJECT sym_alg = { .algorithm = TPM2_ALG_NULL };
 
     /* Symmetric key */
     if (ctx.input_enc_key_file) {
         enc_key.size = 16;
-        result = files_load_bytes_from_path(ctx.input_enc_key_file,
+        bool result = files_load_bytes_from_path(ctx.input_enc_key_file,
                 enc_key.buffer, &enc_key.size);
         if (!result) {
             LOG_ERR("Failed to load symmetric encryption key\"%s\"",
@@ -595,10 +506,14 @@ static tool_rc tpm_import(ESYS_CONTEXT *ectx) {
                     enc_key.size);
             return tool_rc_general_error;
         }
+
+        sym_alg.algorithm = TPM2_ALG_AES;
+        sym_alg.keyBits.aes = 128;
+        sym_alg.mode.aes = TPM2_ALG_CFB;
     }
 
     /* Private key */
-    result = files_load_private(ctx.input_key_file, &duplicate);
+    bool result = files_load_private(ctx.input_key_file, &duplicate);
     if (!result) {
         LOG_ERR("Failed to load duplicate \"%s\"", ctx.input_key_file);
         return tool_rc_general_error;
@@ -640,7 +555,7 @@ static tool_rc tpm_import(ESYS_CONTEXT *ectx) {
         assert(imported_private);
 
         result = files_save_private(imported_private, ctx.private_key_file);
-        free(imported_private);
+        Esys_Free(imported_private);
         if (!result) {
             LOG_ERR("Failed to save private key into file \"%s\"",
                     ctx.private_key_file);
@@ -660,8 +575,8 @@ static tool_rc tpm_import(ESYS_CONTEXT *ectx) {
     if (!result) {
         rc = tool_rc_general_error;
     }
-    return rc;
 
+    return rc;
 }
 
 static tool_rc tpm2_tool_onrun(ESYS_CONTEXT *ectx, tpm2_option_flags flags) {
