@@ -41,6 +41,7 @@ static tool_rc print_nv_public(ESYS_CONTEXT *context, TPMI_RH_NV_INDEX index, TP
     rc = tpm2_tr_get_name(context, tr_handle,
             &name);
     if (rc != tool_rc_success) {
+        free(attrs);
         return rc;
     }
 
@@ -84,7 +85,7 @@ static tool_rc nv_readpublic(ESYS_CONTEXT *context) {
 
     TPMS_CAPABILITY_DATA *capability_data = NULL;
     if (ctx.nv_index == 0) {
-        tool_rc rc = tpm2_getcap(context, TPM2_CAP_HANDLES, tpm2_util_hton_32(TPM2_HT_NV_INDEX),
+        tool_rc rc = tpm2_getcap(context, TPM2_CAP_HANDLES, TPM2_HT_NV_INDEX << 24,
                 TPM2_PT_NV_INDEX_MAX, NULL, &capability_data);
         if (rc != tool_rc_success) {
             return rc;
@@ -128,7 +129,7 @@ static bool on_arg(int argc, char **argv) {
     return on_arg_nv_index(argc, argv, &ctx.nv_index);
 }
 
-bool tpm2_tool_onstart(tpm2_options **opts) {
+static bool tpm2_tool_onstart(tpm2_options **opts) {
 
     *opts = tpm2_options_new(NULL, 0, NULL, NULL,
             on_arg, 0);
@@ -136,9 +137,12 @@ bool tpm2_tool_onstart(tpm2_options **opts) {
     return *opts != NULL;
 }
 
-tool_rc tpm2_tool_onrun(ESYS_CONTEXT *context, tpm2_option_flags flags) {
+static tool_rc tpm2_tool_onrun(ESYS_CONTEXT *context, tpm2_option_flags flags) {
 
     UNUSED(flags);
 
     return nv_readpublic(context);
 }
+
+// Register this tool with tpm2_tool.c
+TPM2_TOOL_REGISTER("nvreadpublic", tpm2_tool_onstart, tpm2_tool_onrun, NULL, NULL)
