@@ -6,6 +6,7 @@
 #include "log.h"
 #include "files.h"
 #include "tpm2.h"
+#include "tpm2_tool.h"
 #include "tpm2_nv_util.h"
 #include "tpm2_options.h"
 #include "tpm2_policy.h"
@@ -183,7 +184,7 @@ static bool on_option(char key, char *value) {
     return result;
 }
 
-bool tpm2_tool_onstart(tpm2_options **opts) {
+static bool tpm2_tool_onstart(tpm2_options **opts) {
 
     const struct option topts[] = {
         { "policy",    required_argument, NULL, 'L'                  },
@@ -223,7 +224,7 @@ bool is_input_option_args_valid(void) {
     return true;
 }
 
-tool_rc tpm2_tool_onrun(ESYS_CONTEXT *ectx, tpm2_option_flags flags) {
+static tool_rc tpm2_tool_onrun(ESYS_CONTEXT *ectx, tpm2_option_flags flags) {
 
     UNUSED(flags);
 
@@ -252,7 +253,10 @@ tool_rc tpm2_tool_onrun(ESYS_CONTEXT *ectx, tpm2_option_flags flags) {
     return tpm2_policy_tool_finish(ectx, ctx.session, ctx.policy_digest_path);
 }
 
-tool_rc tpm2_tool_onstop(ESYS_CONTEXT *ectx) {
+static tool_rc tpm2_tool_onstop(ESYS_CONTEXT *ectx) {
     UNUSED(ectx);
     return tpm2_session_close(&ctx.session);
 }
+
+// Register this tool with tpm2_tool.c
+TPM2_TOOL_REGISTER("policycountertimer", tpm2_tool_onstart, tpm2_tool_onrun, tpm2_tool_onstop, NULL)
