@@ -230,14 +230,6 @@ int main(int argc, char **argv) {
         tpm2_errata_init(ctx.ectx);
     }
 
-    /*
-     * Load the openssl error strings and algorithms
-     * so library routines work as expected.
-     */
-    OpenSSL_add_all_algorithms();
-    OpenSSL_add_all_ciphers();
-    ERR_load_crypto_strings();
-
     /*
      * Call the specific tool, all tools implement this function instead of
      * 'main'.
